#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESConstants
import JESCommandWindowDocument

import java.awt as awt
import java.awt.Toolkit as Toolkit
import java.awt.datatransfer.Clipboard as Clipboard 
import java.awt.datatransfer.Transferable as Transferable
import java.awt.datatransfer.DataFlavor as DataFlavor
import javax.swing.text.JTextComponent as JTextComponent
from JESAction import JESAction

from pawt import colors, swing
from pawt.swing import KeyStroke, text

import string

FALSE = (-1)
TRUE = 1
class JESCommandWindow(swing.JTextPane):

################################################################################
# Function name: __init__
# Parameters:
#     -gui: A reference to the parent JESUI class
# Return:
#     A new instance of the JESCommandWindow class
# Description: 
#     Creates a new instance of JESCommandWindow
################################################################################
    def __init__(self, gui):
        self.program = gui.program
        self.gui = gui
  
        self.setDocument(JESCommandWindowDocument.JESCommandWindowDocument(self))
        self.document = self.getDocument()
        self.setCharacterAttributes(self.document.getTextAttrib(), TRUE) 	
        self.setBackground(colors.black)
        self.setForeground(colors.white)
        self.setCaretColor(colors.white)
        self.setCaretPosition(0)

        parentKeymap = self.getKeymap()
        commandKeymap = self.addKeymap("commandKeymap", parentKeymap)
        commandKeymap.addActionForKeyStroke(KeyStroke.getKeyStroke('\n'), 
                                            makeAction(self.enter))
        self.setKeymap(commandKeymap)

        #This is a flag for JESCommandWindowDocument's insertString method.  It 
        #indicates whether or not JES made the method call.
        self.isSystem = TRUE
        self.setText(">>> ")
        self.isSystem = FALSE

        self.currentPos = self.document.getLength()
        self.oldPos = self.currentPos

        self.my_keymap = commandKeymap

        #Initialize command.  Command holds the line or lines of commands 
        #entered by the user that will be sent to the interpreter
        self.command = ""

        #initialize heldText.  heldText holds the last text to be copied or cut.
        self.heldText = ""

################################################################################
# Function name: getCommandText
# Return:
#     -commandText: a string containing commands for the interpreter
# Description:
#     This function gets all the text in the command window in between the old 
#     cursor position and the new cursor position.
################################################################################
    def getCommandText(self):
        self.currentPos = self.document.getLength()
        command = self.document.getText(self.oldPos, 
                                        self.currentPos - self.oldPos)
        self.oldPos = self.currentPos
        return commandText       

################################################################################
# Function name: showText
# Parameters:
#     -text: output message to display
# Description: 
#     This function displays the text as output from the interpreter.
################################################################################
    def showText(self, text):
        self.isSystem = TRUE
        self.document.insertString(self.currentPos, 
                                   text, 
                                   self.document.getTextAttrib())
        self.isSystem = FALSE
        self.oldPos = self.currentPos
        self.currentPos = self.document.getLength()

################################################################################
# Function name: showError
# Parameters:
#     -text: error message to display
# Description: 
#     This method displays the text as an error thrown from the interpreter.
################################################################################
    def showError(self, text):
        self.isSystem = TRUE
        self.document.insertString(self.currentPos, 
                                   text, 
                                   self.document.getTextAttrib())
        self.isSystem = FALSE
        self.oldPos = self.currentPos
        self.currentPos = self.document.getLength()

################################################################################
# Function name: enter
# Description: 
#     checks to make sure that the enter key was pressed after the last command prompt.  If so, calls 
#     enterHelper().  If not, nothing happens.
#
################################################################################
    def enter(self):
        enteredTextPos = self.getCaretPosition()
        if enteredTextPos >= self.oldPos:
            self.enterHelper()

################################################################################
# Function name: enterHelper
# Description: 
#     This method listens for the enter key to be pressed, and compiles a string 
#     to send to the interpreter that is either single line or multiline.
#     
################################################################################
    def enterHelper(self):
        self.currentPos = self.document.getLength()
        line = self.document.getText(self.oldPos, 
                                     self.currentPos - self.oldPos)
        colon = string.rfind(line, ':')
        ellipsis = string.find(line, '...')

        #first check to see if the line ends with a colon
        if colon != FALSE:

            #then check for an ellipsis at the  beginning of the line
            if ellipsis != FALSE:
                self.showText("... ")
                removeEllipsis = string.split(line, '... ')
                textToAdd = removeEllipsis[1]
                self.command += textToAdd

            else:
                self.showText("... ")
                removePrompt = string.split(line, '>>>')
                self.command = removePrompt[0]

        #if there is no colon:
        else:

            #check to see if the line begins with an ellipsis
            if ellipsis != FALSE:

                newlineCheck = string.split(line, '...')
                #Check to see if it's just a newline, signifying the end of a 
                #multiline sequence
                if newlineCheck[1] == (' \n'):
                    self.runCommand() 
                    #self.setKeymap(None)
                    #self.program.runCommand(self.command)
                    #DNR
                    #self.command = ""
                    #self.showText(">>> ")
                    #self.currentPos = self.document.getLength()
                    #self.oldPos = self.currentPos

                else:
                    self.showText("... ")
                    lengthOfText = len(newlineCheck[1]) + 1
                    textToAdd = string.rjust(newlineCheck[1], lengthOfText)
                    self.command += textToAdd

            #if there's no colon or ellipsis, it's a single line command
            else:
                removePrompt = string.split(line, '>>>')
                self.command = removePrompt[0]
                self.runCommand()
                #self.program.runCommand(self.command )
                #DNR
                #self.command = ""
                #self.showText(">>> ")
                #self.currentPos = self.document.getLength()
                #self.oldPos = self.currentPos

################################################################################
# Function name: runCommand
# Description: 
#     calls JESProgram.runCommand with the code entered by the user
#
################################################################################
    def runCommand(self):
        self.program.runCommand(self.command)

################################################################################
# Function name: restoreConsole
# Description: 
#	Gathers the output of the interpreter and redraws the command window.
#
################################################################################
    def restoreConsole(self,mode):
        self.command = ""
        responseText = self.program.getTextForCommandWindow()

        #if responseText == '':
        #    self.showText( responseText)
        #    self.showText(">>> ")
        #    self.currentPos = self.document.getLength()
        #    self.setCaretPosition( self.currentPos )
        #    self.oldPos = self.currentPos
        if mode == "run":
            if responseText == '':
                self.showText(">>> ")
            else:
                self.showText( responseText)    
                self.showText(">>> ")

            self.oldPos = self.document.getLength()

        else:
            pass
        self.currentPos = self.document.getLength()
        self.setCaretPosition( self.currentPos )
        self.setKeymap(self.my_keymap)
    
################################################################################
# Function name: pasteHelper
# Description:
#	Extracts the first line of a multiline command from the string of text on the clipboard.
#
################################################################################
    def pasteHelper(self):
        #ugly work to get the text from the clipboard
        toolkit = Toolkit.getDefaultToolkit()
        clipboardContents = toolkit.getSystemClipboard().getContents(self)
        flavor = clipboardContents.getTransferDataFlavors()
        clipboardText = clipboardContents.getTransferData(flavor[0])
        #split to remove anything after the newline, if it's multiline
        removeNewline = string.split(clipboardText, '\n')
        self.textToPaste = removeNewline[0]

################################################################################
# Function name: paste
# Description: 
#     This function overrides JESUI's paste function and performs the paste 
#     operation.  If the cursor (or selection) is located after the prompt (and 
#     the line of text has not yet been sent to the interpreter), paste.  If 
#     the cursor (or selection) is located before the prompt, the paste function
#     does nothing (because anything before the most recent prompt is 
#     uneditable).
################################################################################
    def paste(self):
        selection = self.getSelectedText()

        if selection == None:

           insertPoint = self.getCaretPosition()

           #check if cursor is after the last prompt
           if insertPoint >= self.oldPos:
               self.pasteHelper()
               #paste
               self.document.insertString(insertPoint, 
                                          self.textToPaste, 
                                          self.document.getTextAttrib())

           #if not, append text to the end of the text pane
           else:
               self.setCaretPosition(self.document.getLength())
               self.pasteHelper()
               #paste
               self.document.insertString(self.document.getLength(),
                                          self.textToPaste, 
                                          self.document.getTextAttrib())

        else:
            start = self.getSelectionStart()

            #check to see if selection is after the last prompt
            if start >= self.oldPos:
                self.pasteHelper()
                #paste
                self.replaceSelection(self.textToPaste)

            #if not, append text to the end of the text pane
            else:
                self.setCaretPosition(self.document.getLength())
                self.pasteHelper()
                #paste
                self.document.insertString(self.document.getLength(), 
                                           self.textToPaste, 
                                           self.document.getTextAttrib())


################################################################################
# Function name: cut
# Description: 
#     This function overrides JESUI's cut function and performs the cut 
#     operation.  If the text is located after the prompt (and has not yet been 
#     sent to the interpreter), it can be cut.  Otherwise, the cut function 
#     does nothing (because anything before the most recent prompt is 
#     uneditable).
################################################################################
    def cut(self):
        start = self.getSelectionStart()
        if start >= self.oldPos:
            JTextComponent.cut(self)
        else:
            JTextComponent.copy(self)
            
################################################################################
# Function name: undo
# Description: 
#     This function overrides JESUI's undo function and performs the undo 
#     operation.  This function removes all text entered since that most recent 
#     prompt and has not been sent to the interpreter.
################################################################################
    def undo(self): 
        length = self.document.getLength()
        self.document.remove(self.oldPos, (length-self.oldPos))

################################################################################
# Function name: getOldPos
# Return:
#     -oldPos: Returns JESCommandWindow's oldPos variable (which is the 
#              location of the most recent prompt)
# Description: 
#     This function returns the oldPos variable
################################################################################
    def getOldPos(self):
        return self.oldPos

################################################################################
# Function name: getCurrentPos
# Return:
#     -currentPos: Returns JESCommandWindow's currentPos variable                           
# Description: 
#     This function returns the currentPos variable
################################################################################
    def getCurrentPos(self):
        return self.currentPos

################################################################################
# Function name: getIsSystem
# Return:
#     -isSystem: Returns JESCommandWindow's isSystem variable
# Description: 
#     This function returns the isSystem variable
################################################################################
    def getIsSystem(self):
        return self.isSystem

################################################################################
# Function name: makeAction
# Parameters:
#     -obj: an Object to turn into anJESAction
# Return:
#     -obj: either as it entered the function or cast as anJESAction
# Description: 
#     This function determines whether or not the object is anJESAction, and 
#     casts it as anJESAction if necessary.
################################################################################
def makeAction(obj):
        if isinstance(obj,JESAction): return obj
        if callable(obj): return JESAction(obj)