#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESConstants
import JESThread
import sys
import threading
import traceback
import string
import StringIO
from code import compile_command
from java.lang import Thread

class JESInterpreter:
################################################################################
# Function name: __init__
# Parameters:
#     -program - the running instance of JESProgram
# Description: 
#     Creates an instance of JESInterpreter
################################################################################
    def __init__(self, program):
        self.program = program
        self.contextForExecution = {}
        self.interpreterLock = threading.Lock()
        self.isLoading = "false"
        self.preprocessing()

################################################################################
# Function name: preprocessing
# Description: 
#     Loads the preprocessing file into the user's interpreter
################################################################################
    def preprocessing(self):
        try:
            preProcessingFile = open( JESConstants.PRE_PROCESSING_FILE)

            preProcessingText = preProcessingFile.read()
            self.runCommand(preProcessingText,'load')
            return 'true'
        except:    
            self.program.sendTextToCommandWindow(JESConstants.NO_PRE_PROCESSING_FILE_ERR_MSG)
            return ''

################################################################################
# Function name: runCommand
# Parameters:
#     -compiledCode: the user's text, but compiled by the compile_command
#                    command
# Description:
#     Provides external classes the ability to start execution of user text.
#     This method calls gui's setRunning(runBool) method and then sends text to
#     the Jython interpreter. If the command generates an error or output, this
#     method calls sendOutput().
################################################################################
    def runCommand(self, compiledCode, mode = 'run'):
        try:
            jesThread = JESThread.JESThread(compiledCode, self,mode)
            self.jesThread = jesThread


            jesThread.start()
        except:
            self.program.gui.commandWindow.restoreConsole()

################################################################################
# Function name: sendOutput
# Parameters: 
#     responseText: the text to be sent
# Description: 
#     Sends text to the UI where the user can see it
################################################################################
    def sendOutput(self, responseText):
        self.program.sendTextToCommandWindow(responseText)

################################################################################
# FunctionName: sendError
# Description:
#     Called when user code throws an exception.  It sends an error message to
#     the command window.
################################################################################
    def sendError(self):

        exc_type, exc_value, exc_traceback = sys.exc_info()
        msg = self.GetExceptionDescription( exc_type )
        self.sendOutput(msg)

################################################################################
# Function name: stopThread
# Description: 
#     Stops a running thread of user code
################################################################################
    def stopThread(self):
        
        self.jesThread.stop()
        #self.interpreterLock.release()
################################################################################
# Function name: load
# Parameters:
#     -fileText - the text to be loaded
# Description: 
#     Loads the user's code into the context of the command window's
#     interpreter.  reloads the preprocessing code, and if that succeds, loads
#     the user's code.  uses loadHelper to perform the loading.
################################################################################
    def load(self, fileText):
        preprocessingWorked = self.preprocessing()
        if preprocessingWorked != '':
            self.loadHelper( fileText)
        else :
            self.program.sendTextToCommandWindow(JESConstands.NO_PRE_PROCESSING_FILE_ERR_MSG)

################################################################################
# Function name: loadHelper
# Parameters:
#     -fileText: the text of the file to be loaded
# Description: 
#     Performs the actual loading of the file into the context of the
#     user's command window
################################################################################
    def loadHelper(self, fileText):
        try:
            # technology issue:
            # compile_command only checks syntax
            # when called on a definition - it does not produce any code
            # we still pass fileText to runCommand     
            code = compile_command( string.join(fileText,'\n') )

        except SyntaxError:       
            sendError()
        else:
            self.runCommand(fileText,'load')

################################################################################
# Function name: GetExceptionDescription
# Parameters:
#     -exception: exception object containing information about the exception
#                 that occured.
# Return:
#     Returns a message as a string describing the error that occured.
# Description:
#     This function takes in an exception object and returns a user friendly
#     message describing the error that occured.
################################################################################
    def GetExceptionDescription(self, exception):
        className = exception.__class__.__name__

        if JESConstants.EXCEPTION_MESSAGES.has_key(className):
            msg = JESConstants.EXCEPTION_MESSAGES[className]% exception.__dict__
        else:
            msg = JESConstants.GENERIC_EXCEPTION_MESSAGE + exception.__doc__

            try:
                for eachArg in exception.args:
                    msg += '\n' + eachArg
            except:
                # the try/except here is to handle the case where the exception
                # does not have any arguments.  The exception is thrown, and
                # an empty message is sent.               
                msg += '\n'

        return msg

################################################################################
# Function name: getIsLoading
# Description:
#     
################################################################################
    def getIsLoading(self):
        return self.isLoading
