#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESStdOutputBuffer
import threading
import traceback
import StringIO
import sys
from java.lang import Thread

class JESThread(Thread):
################################################################################
# Function name: __init__
# Parameters:
#     -code: string of code that will be executed.
#     -interpreter: instance of the JESInterpretor object that is creating this
#                   object.
# Return:
#     An instance of the JESThread class.
# Description:
#     Creates a new instance of the JESThread class
################################################################################
    def __init__(self, code, interpreter,mode):
        self.mode = mode
        self.code = code
        self.interpreter = interpreter
        self.contextForExecution = interpreter.contextForExecution

################################################################################
# Function name: run
# Description:
#     Begins running the Jython code that was passed into the JESThread instance
#     upon creation.  The code will run until all of it has been executed,
#     an exeption occurs, or stopPython is called.
################################################################################
    def run(self):
        
        #print "JESTHREAD:RUN:TRY"
        self.interpreter.interpreterLock.acquire()
        #print "JESTHREAD:RUN:GOT"

        self.interpreter.program.gui.setRunning(1)
        errMsg = ''
        outputBuffer = JESStdOutputBuffer.JESStdOutputBuffer(sys.stdout)

        try:
            exec self.code in self.contextForExecution
            
            #Include these lines to actually exit on a sys.exit() call
            #except SystemExit, value:
            #   raise SystemExit, value

        except:
            errFp = StringIO.StringIO()
            
            exc_type, exc_value, exc_traceback = sys.exc_info()
            l = len(traceback.extract_tb(sys.exc_traceback)) 
            try:
                1/0
            except:
                m = len(traceback.extract_tb(sys.exc_traceback))
                traceback.print_exception(exc_type,
                                          exc_value,
                                          exc_traceback,
                                          l-m,
                                          file=errFp)
                errMsg = errFp.getvalue()

        outputBuffer.restoreOutput()
        self.interpreter.sendOutput( errMsg )
        self.interpreter.sendOutput( outputBuffer.getText() )


        self.interpreter.program.gui.commandWindow.restoreConsole(self.mode)
        self.interpreter.program.gui.setRunning(0)
       
        #print "JESTHREAD:RUN:release"
        self.interpreter.interpreterLock.release()
        #print self.contextForExecution

################################################################################
# Function name: stopPython
# Description:
#     Stops the interpreter (should be used only after the run method is
#     called).
################################################################################
    def stopPython(self):

        self.interpreter.program.gui.commandWindow.restoreConsole(self.mode)
        
        self.interpreter.sendOutput( outputBuffer.getText() )
        self.interpreter.program.gui.setRunning(0)
        #print "JESTHREAD:STOPPYTHON:RELEASE"
        self.interpreter.interpreterLock.release()
        self.stop()
